@@ -66,6 +66,9 @@ lavData <- function(data              = NULL,          # data.frame
     if(is.null(warn)) {
         warn <- TRUE
     }
+    if(allow.single.case) { # eg, in lavPredict
+        warn <- FALSE
+    }
 
     # four scenarios:
     #    0) data is already a lavData object: do nothing
@@ -192,9 +195,9 @@ lavData <- function(data              = NULL,          # data.frame
         }
 
         # get ov.names
-        if (is.null(ov.names)) {
+        if(is.null(ov.names)) {
             ov.names <- lapply(sample.cov, row.names)
-        } else if (!is.list(ov.names)) {
+        } else if(!is.list(ov.names)) {
             # duplicate ov.names for each group
             tmp <- ov.names; ov.names <- vector("list", length = ngroups)
             ov.names[1:ngroups] <- list(tmp)
@@ -242,6 +245,24 @@ lavData <- function(data              = NULL,          # data.frame
             warning("lavaan WARNING: std.ov argument is ignored if only sample statistics are provided.")
         }
 
+        # check variances (new in 0.6-7)
+        for(g in seq_len(ngroups)) {
+            VAR <- diag(sample.cov[[g]])
+            # 1. finite?
+            if(!all(is.finite(VAR))) {
+                stop("lavaan ERROR: at least one variance in the sample covariance matrix is not finite.")
+            }
+            # 2. near zero (or negative)?
+            if(any(VAR < .Machine$double.eps)) {
+                stop("lavaan ERROR: at least one variance in the sample covariance matrix is (near) zero or negative.")
+            }
+            # 3. very large?
+            max.var <- max(VAR)
+            if(max.var > 1000000) {
+                warning("lavaan WARNING: some observed variances in the sample covariance matrix are larger than 1000000.")
+            }
+        }
+
         # construct lavData object
         lavData <- new("lavData",
                        data.type   = "moment",
@@ -368,10 +389,13 @@ lavData <- function(data              = NULL,          # data.frame
         Lp <- vector("list", length = ngroups)
         for(g in 1:ngroups) {
             if(nlevels > 1L) {
+                # ALWAYS add ov.names.x at the end, even if conditional.x
+                OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
                 Lp[[g]] <- lav_data_cluster_patterns(Y = NULL, clus = NULL,
                                                  cluster = cluster,
                                                  multilevel = TRUE,
-                                                 ov.names = ov.names[[g]],
+                                                 ov.names = OV.NAMES,
+                                                 ov.names.x = ov.names.x[[g]],
                                                  ov.names.l = ov.names.l[[g]])
             }
         } # g
@@ -648,7 +672,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
     }
     # check for zero-cases
     idx <- which(ov$nobs == 0L | ov$var == 0)
-    if(length(idx) > 0L) {
+    if(!allow.single.case && length(idx) > 0L) {
         OV <- as.data.frame(ov)
         rn <- rownames(OV)
         rn[idx] <- paste(rn[idx], "***", sep="")
@@ -668,7 +692,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
     }
     # check for ordered variables with only 1 level
     idx <- which(ov$type == "ordered" & ov$nlev == 1L)
-    if(length(idx) > 0L) {
+    if(!allow.single.case && length(idx) > 0L) {
         OV <- as.data.frame(ov)
         rn <- rownames(OV)
         rn[idx] <- paste(rn[idx], "***", sep="")
@@ -720,7 +744,13 @@ lav_data_full <- function(data          = NULL,          # data.frame
     for(g in 1:ngroups) {
 
         # extract variables in correct order
-        ov.idx  <- ov$idx[match(ov.names[[g]],   ov$name)]
+        if(!is.null(cluster) && length(cluster) > 0L) {
+            # keep 'joint' (Y,X) matrix in @X if multilevel (or always?)
+            OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
+            ov.idx  <- ov$idx[match(OV.NAMES,   ov$name)]
+        } else {
+            ov.idx  <- ov$idx[match(ov.names[[g]],   ov$name)]
+        }
         exo.idx <- ov$idx[match(ov.names.x[[g]], ov$name)]
         all.idx <- unique(c(ov.idx, exo.idx))
 
@@ -831,26 +861,6 @@ lav_data_full <- function(data          = NULL,          # data.frame
             }
         }
 
-        # missing data
-        if(missing != "listwise") {
-            # get missing patterns
-            Mp[[g]] <- lav_data_missing_patterns(X[[g]],
-                           sort.freq = TRUE, coverage = TRUE)
-            # checking!
-            if(length(Mp[[g]]$empty.idx) > 0L) {
-                # new in 0.6-4: we return 'original' index in full data.frame
-                empty.case.idx <- case.idx[[g]][ Mp[[g]]$empty.idx ]
-                if(warn) {
-                    warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
-                }
-            }
-            if(warn && any(Mp[[g]]$coverage < 0.1)) {
-                warning("lavaan WARNING: due to missing values, some pairwise combinations have less than 10% coverage")
-            }
-            # in case we had observations with only missings
-            nobs[[g]] <- NROW(X[[g]]) - length(Mp[[g]]$empty.idx)
-        }
-
         # response patterns (ordered variables only)
         ord.idx <- which(ov.names[[g]] %in% ov$name[ov$type == "ordered"])
         if(length(ord.idx) > 0L) {
@@ -865,6 +875,43 @@ lav_data_full <- function(data          = NULL,          # data.frame
             warning("lavaan WARNING: small number of observations (nobs < nvar)", txt,
                     "\n  nobs = ", nobs[[g]], " nvar = ", nvar)
         }
+        # check variances per group (if we have multiple groups)
+        # to catch zero-variance variables within a group (new in 0.6-8)
+        if(ngroups > 1L) {
+            # X
+            group.var <- apply(X[[g]], 2, var, na.rm = TRUE)
+            zero.var <- which(group.var < .Machine$double.eps)
+            if(length(zero.var) == 0L) {
+                # all is good
+            } else {
+                # some zero variances!
+                gtxt <- if(ngroups > 1L) {
+                            paste(" in group ", g, ":", sep = "")
+                        } else { ":" }
+                txt <- c("some variables have no variance ", gtxt,
+                   "\n",
+                   paste( ov.names[[g]][zero.var], collapse = " "))
+                stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+            }
+
+            # eXo (if conditional.x = TRUE)...
+            if(length(exo.idx) > 0L) {
+                group.var <- apply(eXo[[g]], 2, var, na.rm = TRUE)
+                zero.var <- which(group.var < .Machine$double.eps)
+                if(length(zero.var) == 0L) {
+                    # all is good
+                } else {
+                    # some zero variances!
+                    gtxt <- if(ngroups > 1L) {
+                                paste(" in group ", g, ":", sep = "")
+                            } else { ":" }
+                    txt <- c("some exogenous variables have no variance ", gtxt,
+                       "\n",
+                       paste( ov.names.x[[g]][zero.var], collapse = " "))
+                    stop(lav_txt2message(txt, header = "lavaan ERROR:"))
+                }
+            }
+        }
 
         # cluster information
         if(length(cluster) > 0L) {
@@ -875,10 +922,13 @@ lav_data_full <- function(data          = NULL,          # data.frame
             } else {
                 multilevel <- FALSE
             }
+            # ALWAYS add ov.names.x at the end, even if conditional.x (0.6-7)
+            OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
             Lp[[g]] <- lav_data_cluster_patterns(Y = X[[g]], clus = clus,
                                                  cluster = cluster,
                                                  multilevel = multilevel,
-                                                 ov.names = ov.names[[g]],
+                                                 ov.names = OV.NAMES,
+                                                 ov.names.x = ov.names.x[[g]],
                                                  ov.names.l = ov.names.l[[g]])
 
             # new in 0.6-4
@@ -967,6 +1017,37 @@ lav_data_full <- function(data          = NULL,          # data.frame
 
         } # clustered data
 
+        # missing data
+        if(missing != "listwise") {
+
+            if(length(cluster) > 0L) {
+                # get missing patterns
+                Mp[[g]] <- lav_data_missing_patterns(X[[g]],
+                               sort.freq = TRUE, coverage = TRUE,
+                               Lp = Lp[[g]])
+            } else {
+                # get missing patterns
+                Mp[[g]] <- lav_data_missing_patterns(X[[g]],
+                               sort.freq = TRUE, coverage = TRUE,
+                               Lp = NULL)
+            }
+
+            # checking!
+            if(length(Mp[[g]]$empty.idx) > 0L) {
+                # new in 0.6-4: return 'original' index in full data.frame
+                empty.case.idx <- case.idx[[g]][ Mp[[g]]$empty.idx ]
+                if(warn) {
+                    warning("lavaan WARNING: some cases are empty and will be ignored:\n  ", paste(empty.case.idx, collapse=" "))
+                }
+            }
+            if(warn && any(Mp[[g]]$coverage < 0.1)) {
+                warning("lavaan WARNING: due to missing values, some pairwise combinations have less than 10% coverage")
+            }
+            # in case we had observations with only missings
+            nobs[[g]] <- NROW(X[[g]]) - length(Mp[[g]]$empty.idx)
+    } # missing
+
+
     } # groups, at first level
 
     # sampling weigths, again
@@ -1027,353 +1108,5 @@ lav_data_full <- function(data          = NULL,          # data.frame
     lavData
 }
 
-# get missing patterns
-lav_data_missing_patterns <- function(Y, sort.freq = FALSE, coverage = FALSE) {
 
-    # construct TRUE/FALSE matrix: TRUE if value is observed
-    OBS <- !is.na(Y)
-
-    # empty cases
-    empty.idx <- which(rowSums(OBS) == 0L)
-
-    # this is what we did in < 0.6
-    #if(length(empty.idx) > 0L) {
-    #    OBS <- OBS[-empty.idx,,drop = FALSE]
-    #}
-
-    # pattern of observed values per observation
-    case.id <- apply(1L * OBS, 1L, paste, collapse = "")
-
-    # remove empty patterns
-    if(length(empty.idx)) {
-        case.id.nonempty <- case.id[-empty.idx]
-    } else {
-        case.id.nonempty <- case.id
-    }
-
-    # sort non-empty patterns (from high occurence to low occurence)
-    if(sort.freq) {
-        TABLE <- sort(table(case.id.nonempty), decreasing = TRUE)
-    } else {
-        TABLE <- table(case.id.nonempty)
-    }
-
-    # unique pattern ids
-    pat.id <- names(TABLE)
-
-    # number of patterns
-    pat.npatterns  <- length(pat.id)
-
-    # case idx per pattern
-    pat.case.idx <- lapply(seq_len(pat.npatterns),
-                           function(p) which(case.id == pat.id[p]))
-
-    # unique pattern frequencies
-    pat.freq <- as.integer(TABLE)
-
-    # first occurrence of each pattern
-    pat.first <- match(pat.id, case.id)
-
-    # TRUE/FALSE for each pattern
-    pat.obs <- OBS[pat.first,,drop = FALSE] # observed per pattern
-
-    Mp <- list(npatterns = pat.npatterns, id = pat.id, freq = pat.freq,
-               case.idx = pat.case.idx, pat = pat.obs, empty.idx = empty.idx)
-
-    if(coverage) {
-        # FIXME: if we have empty cases, include them in N?
-        # no for now
-        Mp$coverage <- crossprod(OBS) / sum(pat.freq)
-        #Mp$coverage <- crossprod(OBS) / NROW(Y)
-    }
-
-    Mp
-}
-
-# get response patterns (ignore empty cases!)
-lav_data_resp_patterns <- function(Y) {
-
-    # construct TRUE/FALSE matrix: TRUE if value is observed
-    OBS <- !is.na(Y)
-
-    # empty cases
-    empty.idx <- which(rowSums(OBS) == 0L)
-
-    # removeYempty cases
-    if(length(empty.idx) > 0L) {
-        Y <- Y[-empty.idx,,drop = FALSE]
-    }
-
-    ntotal <- nrow(Y); nvar <- ncol(Y)
-
-    # identify, label and sort response patterns
-    id <- apply(Y, MARGIN = 1, paste, collapse = "")
-
-    # sort patterns (from high occurence to low occurence)
-    TABLE <- sort(table(id), decreasing = TRUE)
-    order <- names(TABLE)
-    npatterns <- length(TABLE)
-    pat <- Y[match(order, id), , drop = FALSE]
-    row.names(pat) <- as.character(TABLE)
-
-    # handle NA?
-    Y[is.na(Y)] <- -9
-    total.patterns <- prod(apply(Y, 2, function(x) length(unique(x))))
-    empty.patterns <- total.patterns - npatterns
-    # return a list
-    #out <- list(nobs=ntotal, nvar=nvar,
-    #            id=id, npatterns=npatterns,
-    #            order=order, pat=pat)
-
-    # only return pat
-    out <- list(npatterns=npatterns, pat=pat, total.patterns=total.patterns,
-                empty.patterns=empty.patterns)
-
-    out
-}
-
-# get cluster information
-# - cluster can be a vector!
-# - clus can contain multiple columns!
-lav_data_cluster_patterns <- function(Y = NULL,
-                                      clus = NULL,    # the cluster ids
-                                      cluster = NULL, # the cluster 'names'
-                                      multilevel = FALSE,
-                                      ov.names, ov.names.l) {
-
-    # how many levels?
-    nlevels <- length(cluster) + 1L
-
-    # did we get any data (or is this just for simulateData)
-    if(!is.null(Y) && !is.null(clus)) {
-        haveData <- TRUE
-    } else {
-        haveData <- FALSE
-    }
-
-    # check clus
-    if(haveData) {
-        stopifnot(ncol(clus) == (nlevels - 1L), nrow(Y) == nrow(clus))
-    }
-
-    cluster.size    <- vector("list", length = nlevels)
-    cluster.id      <- vector("list", length = nlevels)
-    cluster.idx     <- vector("list", length = nlevels)
-    nclusters       <- vector("list", length = nlevels)
-    cluster.sizes   <- vector("list", length = nlevels)
-    ncluster.sizes  <- vector("list", length = nlevels)
-    cluster.size.ns <- vector("list", length = nlevels)
-    ov.idx          <- vector("list", length = nlevels)
-    both.idx        <- vector("list", length = nlevels)
-    within.idx      <- vector("list", length = nlevels)
-    between.idx     <- vector("list", length = nlevels)
-    both.names      <- vector("list", length = nlevels)
-    within.names    <- vector("list", length = nlevels)
-    between.names   <- vector("list", length = nlevels)
-
-    # level-1 is special
-    if(haveData) {
-        nclusters[[1]] <- NROW(Y)
-    }
-
-    if(multilevel) {
-        ov.idx[[1]] <- match(ov.names.l[[1]], ov.names)
-    }
-
-    # for the remaining levels...
-    for(l in 2:nlevels) {
-        if(haveData) {
-            CLUS <- clus[,(l-1L)]
-            cluster.id[[l]]      <- unique(CLUS)
-            cluster.idx[[l]]     <- match(CLUS, cluster.id[[l]])
-            cluster.size[[l]]    <- tabulate(cluster.idx[[l]])
-            nclusters[[l]]       <- length(cluster.size[[l]])
-            cluster.sizes[[l]]   <- unique(cluster.size[[l]])
-            ncluster.sizes[[l]]  <- length(cluster.sizes[[l]])
-            cluster.size.ns[[l]] <- as.integer(table(factor(cluster.size[[l]],
-                                     levels = as.character(cluster.sizes[[l]]))))
-        } else {
-            cluster.id[[l]]      <- integer(0L)
-            cluster.idx[[l]]     <- integer(0L)
-            cluster.size[[l]]    <- integer(0L)
-            nclusters[[l]]       <- integer(0L)
-            cluster.sizes[[l]]   <- integer(0L)
-            ncluster.sizes[[l]]  <- integer(0L)
-            cluster.size.ns[[l]] <- integer(0L)
-        }
-
-        if(multilevel) {
-            # index of ov.names for this level
-            ov.idx[[l]]         <- match(ov.names.l[[l]], ov.names)
-
-            both.idx[[l]]       <- which( ov.names %in% ov.names.l[[1]] &
-                                          ov.names %in% ov.names.l[[2]])
-            within.idx[[l]]     <- which( ov.names %in% ov.names.l[[1]] &
-                                         !ov.names %in% ov.names.l[[2]])
-            between.idx[[l]]    <- which(!ov.names %in% ov.names.l[[1]] &
-                                          ov.names %in% ov.names.l[[2]])
-
-            # names
-            both.names[[l]]     <- ov.names[ ov.names %in% ov.names.l[[1]] &
-                                             ov.names %in% ov.names.l[[2]] ]
-            within.names[[l]]   <- ov.names[ ov.names %in% ov.names.l[[1]] &
-                                            !ov.names %in% ov.names.l[[2]] ]
-            between.names[[l]]  <- ov.names[!ov.names %in% ov.names.l[[1]] &
-                                             ov.names %in% ov.names.l[[2]] ]
-        }
-    }
-
-    out <- list(cluster = cluster, # clus = clus,
-                # per level
-                nclusters = nclusters,
-                cluster.size = cluster.size, cluster.id = cluster.id,
-                cluster.idx = cluster.idx, cluster.sizes = cluster.sizes,
-                ncluster.sizes = ncluster.sizes,
-                cluster.size.ns = cluster.size.ns,
-                ov.idx = ov.idx, both.idx = both.idx, within.idx = within.idx,
-                between.idx = between.idx,
-                both.names = both.names, within.names = within.names,
-                between.names = between.names)
-
-    out
-}
-
-setMethod("show", "lavData",
-function(object) {
-    # print 'lavData' object
-    lav_data_print_short(object)
-})
-
-lav_data_print_short <- function(object, nd = 3L) {
-
-    lavdata <- object
-    num.format  <- paste("%", max(8L, nd + 5L), ".", nd, "f", sep = "")
-
-    # listwise deletion?
-    listwise <- twocolumn <- FALSE
-    for(g in 1:lavdata@ngroups) {
-       if(lavdata@nobs[[1L]] != lavdata@norig[[1L]]) {
-           listwise <- twocolumn <- TRUE
-           break
-       }
-    }
-
-    # header? no, for historical reasons only
-    #cat("Data information:\n\n")
-
-    c1 <- c2 <- c3 <- character(0L)
-
-    # number of observations
-    if(lavdata@ngroups == 1L) {
-        if(listwise) {
-            c1 <- c(c1, ""); c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
-        }
-        c1 <- c(c1, "Number of observations")
-        c2 <- c(c2, lavdata@nobs[[1L]])
-        c3 <- c(c3, ifelse(listwise, lavdata@norig[[1L]], ""))
-    } else {
-        c1 <- c(c1, "Number of observations per group:");
-        if(listwise) {
-            c2 <- c(c2, "Used"); c3 <- c(c3, "Total")
-        } else {
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-        }
-        for(g in 1:lavdata@ngroups) {
-            c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-            c2 <- c(c2, lavdata@nobs[[g]])
-            c3 <- c(c3, ifelse(listwise, lavdata@norig[[g]], ""))
-        } # g
-    }
-
-    # number of clusters
-    if(lavdata@ngroups == 1L) {
-        if( (.hasSlot(lavdata, "nlevels")) && # in case we have an old obj
-            (lavdata@nlevels > 1L) ) {
-            for(l in 2:lavdata@nlevels) {
-                c1 <- c(c1,
-                        paste("Number of clusters [", lavdata@cluster[l-1], "]",
-                        sep = ""))
-                c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[l]])
-                c3 <- c(c3, "")
-            }
-        } else if( (.hasSlot(lavdata, "cluster")) &&
-                   (length(lavdata@cluster) > 0L) ) {
-            c1 <- c(c1, paste("Number of clusters [", lavdata@cluster, "]",
-                              sep = ""))
-            c2 <- c(c2, lavdata@Lp[[1]]$nclusters[[2]])
-            c3 <- c(c3, "")
-        }
-    } else {
-        if( (.hasSlot(lavdata, "nlevels")) && (lavdata@nlevels > 1L) ) {
-            for(l in 2:lavdata@nlevels) {
-                c1 <- c(c1,
-                  paste("Number of clusters [", lavdata@cluster[l-1], "]:",
-                        sep = ""))
-                c2 <- c(c2, ""); c3 <- c(c3, "")
-                for(g in 1:lavdata@ngroups) {
-                    c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                    c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[l]])
-                    c3 <- c(c3, "")
-                }
-            }
-        } else if( (.hasSlot(lavdata, "cluster")) &&
-               (length(lavdata@cluster) > 0L) ) {
-            c1 <- c(c1,
-             paste("Number of clusters [", lavdata@cluster, "]:", sep = ""))
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-            for(g in 1:lavdata@ngroups) {
-                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                c2 <- c(c2, lavdata@Lp[[g]]$nclusters[[2]])
-                c3 <- c(c3, "")
-            }
-        }
-    }
-
-    # missing patterns?
-    if(!is.null(lavdata@Mp[[1L]])) {
-        if(lavdata@ngroups == 1L) {
-            c1 <- c(c1, "Number of missing patterns")
-            c2 <- c(c2, lavdata@Mp[[1L]]$npatterns)
-            c3 <- c(c3, "")
-        } else {
-            c1 <- c(c1, "Number of missing patterns per group:")
-            c2 <- c(c2, ""); c3 <- c(c3, "")
-            for(g in 1:lavdata@ngroups) {
-                c1 <- c(c1, sprintf("  %-40s", lavdata@group.label[[g]]))
-                c2 <- c(c2, lavdata@Mp[[g]]$npatterns)
-                c3 <- c(c3, "")
-            }
-        }
-    }
-
-    # sampling weights?
-    if( (.hasSlot(lavdata, "weights")) && # in case we have an old object
-        (!is.null(lavdata@weights[[1L]])) ) {
-        c1 <- c(c1, "Sampling weights variable")
-        c2 <- c(c2, lavdata@sampling.weights)
-        c3 <- c(c3, "")
-    }
-
-    # empty row
-    c1 <- c(c1, ""); c2 <- c(c2, ""); c3 <- c(c3, "")
-
-    # format c1/c2
-    c1 <- format(c1, width = 43L)
-    c2 <- format(c2, width = 8L + max(0, (nd - 3L)) * 4L, justify = "right")
-    c3 <- format(c3, width = 8L + nd, justify = "right")
-
-    # create character matrix
-    if(twocolumn) {
-        M <- cbind(c1, c2, c3, deparse.level = 0)
-    } else {
-        M <- cbind(c1, c2, deparse.level = 0)
-    }
-    colnames(M) <- rep("",  ncol(M))
-    rownames(M) <- rep(" ", nrow(M))
-
-    # print
-    write.table(M, row.names = TRUE, col.names = FALSE, quote = FALSE)
-
-    invisible(M)
-}
 
