@@ -27,9 +27,9 @@ lav_model_loglik <- function(lavdata        = NULL,
             }
         }
         # nlevels + fiml
-        if(lavdata@nlevels > 1L && lavsamplestats@missing.flag) {
-            logl.ok <- FALSE
-        }
+        #if(lavdata@nlevels > 1L && lavsamplestats@missing.flag) {
+        #    logl.ok <- FALSE
+        #}
     }
 
     # lavsamplestats filled in? (not if no data...)
@@ -54,12 +54,13 @@ lav_model_loglik <- function(lavdata        = NULL,
                 if(lavsamplestats@missing.flag) {
                     logl.group[g] <-
                         lav_mvnorm_cluster_missing_loglik_samplestats_2l(
-                            Yp = lavsamplestats@missing[[g]],
+                            Y1 = lavdata@X[[g]],
                             Y2 = lavsamplestats@YLp[[g]][[2]]$Y2,
                             Lp = lavdata@Lp[[g]],
                             Mp = lavdata@Mp[[g]],
                             Mu.W = Mu.W, Sigma.W = Sigma.W,
                             Mu.B = Mu.B, Sigma.B = Sigma.B,
+                            loglik.x = lavsamplestats@YLp[[g]][[2]]$loglik.x,
                             log2pi = TRUE, minus.two = FALSE)
                 } else {
                     logl.group[g] <- lav_mvnorm_cluster_loglik_samplestats_2l(
